#import required python modules
import numpy as np
import time
import atomwrite

#import compiled fortran library
import FinProjLib

#import fian project python Library
import FinalProjectLibrary as FPL

#get start time for code timings
tic = time.time()

#========== GLOBAL VARIABLES ==========
#distance cutoff for pairwise interactions
Cut = 2.5
#target number of simulated atoms (may vary due to chain length and dispersity)
Ntarg = 3.
#number of simulated chains
Nc = 20
#number density of simulation box
rho = 0.8
#target system temperature
T = 1.0
#set the frequency of velocity rescaling
RescaleFreq = 50
#line search tolerance
EFracTolLS = 1.e-8
#conjugate gradient tolerance
EFracTolCG = 1.e-10
#line search initial step size
dx = 1.e-3

#========== Get Movie Trajectory==========
#set simulated polymer degrees of polymerization
M = 10
#set time step size
dt = 0.001
#set sample frequency
SampleFreq = 50
#set number of equilibration and production steps
NStepsEquil1 = int(1e4)
NStepsEquil2 = int(1e4)
NStepsProd = int(1e5)
Nblocksprod=int(np.floor(NStepsProd/SampleFreq))+1

#assign bonds
BondPairs,LJPairs,N,ChainEnds = FPL.AssignBondsDisperse(Nc,M)

#set simulation box size
L = np.cbrt(N/rho)

#get list of unique chain lenths
Mlist = []
startlist = []
stoplist = []
for i in range(len(ChainEnds)):
    if i == 0:
        start = 0
        stop = ChainEnds[i]
        M = stop - start
    else:
        start = ChainEnds[i-1]+1
        stop = ChainEnds[i]
        M = stop - start + 1
    if not M in Mlist:
        Mlist.append(M)
        startlist.append(start)
        stoplist.append(stop)

#Assign unique species names to one chain or each length
Namelist = ["H","O","N","S","P","Cl","Br","I","Fe","Co","Ni"]
AtomNames = ["C"]*(startlist[0])+[Namelist[0]]*(stoplist[1]-startlist[0]+1)
for i in range(1,len(Mlist)):
    AtomNames += ["C"]*(startlist[i]-stoplist[i-1]-1)+[Namelist[i]]*(stoplist[i]-startlist[i]+1)
#open pdb file to write animation
DisperseLJPolyAnim = atomwrite.pdbfile("DisperseLJPolyAnim2.pdb",L,AtomNames = AtomNames)


#get initial positions
Pos0 = FPL.InitPositions(N,L)
#get initial velocities
Vel = FPL.InitVelocities(N,T)

#relax initial positions to a local minimum
PEnergy, Pos = FPL.ConjugateGradient(Pos0,dx,EFracTolLS,EFracTolCG,L,Cut,BondPairs,LJPairs)
#get initial accelerations
Accel = FPL.InitAccel(Pos,L,Cut,BondPairs,LJPairs)

print "Equilibrating 1/2, M = %.0i, Elapsed Time = %.0f s" % (M,time.time()-tic)

#perform MD equilibration run for the specified number of timesteps
for i in range(1,NStepsEquil1+1):
    
    #step forward by one block, taking RescaleFreq steps
    Pos,Vel,Accel,KEnergy,PEnergy = FinProjLib.vvintegrate(Pos,Vel,Accel,L,Cut,dt,BondPairs,LJPairs)
    
    #rescale velocities every RescaleFreq steps
    if i % RescaleFreq == 0:    
        Vel = FPL.RescaleVelocities(Vel,T)
    
#preallocate (NStepsEquil2+1,2) array to store the PEs and KEs for 
#calculation of the average total energy
Elist = np.zeros((NStepsEquil2+1,2),dtype=float)
Elist[0,0] = KEnergy
Elist[0,1] = PEnergy
    
print "Equilibrating 2/2, M = %.0i, Elapsed Time = %.0f s" % (M,time.time()-tic)

#perform MD equilibration run for the specified number of timesteps
for i in range(1,NStepsEquil2+1):
    
    #step forward by one block, taking RescaleFreq steps
    Pos,Vel,Accel,Elist[i,0],Elist[i,1] = FinProjLib.vvintegrate(Pos,Vel,Accel,L,Cut,dt,BondPairs,LJPairs)
    
    #rescale velocities every RescaleFreq steps
    if i % RescaleFreq == 0:    
        Vel = FPL.RescaleVelocities(Vel,T)
    
#compute mean PEnergy from second equilibration
EAvg = np.mean(np.sum(Elist,axis=1))
#compute target kinetic energy
PEnergy = Elist[-1,1]
KEtarg = EAvg - PEnergy
#rescale velocities to net zero momentum
#Vel = Vel - Vel.mean(axis=0)
#find the total kinetic energy and rescale velocities
KEnergy = 0.5 * np.einsum('ij,ij',Vel,Vel)
VScale = np.sqrt(KEtarg/KEnergy)
Vel = Vel * VScale

#store initial position to calculate MSD
Posinit = Pos.copy()

print "Producing, M = %.0i, Elapsed Time = %.0f s" % (M,time.time()-tic)

#perform MD equilibration run for the specified number of timesteps
for i in range(1,NStepsProd+1):
    
    #step forward by one block, taking SampleFreq steps
    Pos,Vel,Accel,_,_ = FinProjLib.vvintegrate(Pos,Vel,Accel,L,Cut,dt,BondPairs,LJPairs)

    #write positon to animation pdb file
    DisperseLJPolyAnim.write(Pos)

DisperseLJPolyAnim.close()
print "Done! M = %.0i, Elapsed Time = %.0f s" % (M,time.time()-tic)